package my.package;

import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jms.Connection;
import javax.jms.ExceptionListener;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.Session;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.core.config.impl.ConfigurationImpl;
import org.hornetq.core.management.impl.HornetQServerControlImpl;
import org.hornetq.core.remoting.impl.invm.InVMAcceptorFactory;
import org.hornetq.core.remoting.impl.invm.InVMConnectorFactory;
import org.hornetq.core.server.HornetQServer;
import org.hornetq.jms.client.HornetQJMSConnectionFactory;

import static org.hornetq.api.jms.HornetQJMSClient.createConnectionFactory;
import static org.hornetq.core.server.HornetQServers.newHornetQServer;
import static org.hornetq.core.server.JournalType.NIO;

@SuppressWarnings("serial")
public class LaunchSimple {
    private LaunchSimple() {}

    public static void main(String[] args) throws Exception {
        System.out.println("Simplified");

        ConfigurationImpl configurationImpl = new ConfigurationImpl() {{
            setPersistenceEnabled(false);
            setSecurityEnabled(false);
            setJournalType(NIO);
            setClusterPassword("set this to prevent HornetQ warning that it hasn't been set");

            setThreadPoolMaxSize(100);
            setScheduledThreadPoolMaxSize(10);

            Set<TransportConfiguration> transports =
                new HashSet<TransportConfiguration>() {{
                    add(new TransportConfiguration(InVMAcceptorFactory.class.getName()));
                }};
            setAcceptorConfigurations(transports);
        }};

        HornetQServer server = newHornetQServer(configurationImpl);
        server.start();
        HornetQServerControlImpl serverControl = server.getHornetQServerControl();
        serverControl.deployQueue("jms.queue.centralInput", "jms.queue.centralInput", null, false);

        HornetQJMSConnectionFactory connectionFactory =
            createConnectionFactory(new TransportConfiguration(InVMConnectorFactory.class.getName()));

        connectionFactory.setThreadPoolMaxSize(100);
        connectionFactory.setScheduledThreadPoolMaxSize(100);
        connectionFactory.setUseGlobalPools(false);

        connectionFactory.setConsumerWindowSize(0);
        connectionFactory.setConnectionTTL(2000);
        connectionFactory.setClientFailureCheckPeriod(100);

        final Connection connection = connectionFactory.createConnection();
        connection.start();

        connection.setExceptionListener(new ExceptionListener() {
            @Override
            public void onException(JMSException exception) {
                try {
                    log.log(Level.SEVERE, "Exception in Connection", exception);
                    log.log(Level.SEVERE, "Exception in Connection (" + connection.getClientID() + ")", exception);
                } catch (JMSException ex) {
                    log.log(Level.SEVERE, "Exception getting client id in exception", ex);
                }
            }
        });
        Session session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
        Queue queue = session.createQueue("centralInput");

        MessageConsumer consumer = session.createConsumer(queue);
        consumer.setMessageListener(new MessageListenerImplementation(session, connection));

        System.out.println("New: Just hangin' around...");
        while (true) {
            Thread.sleep(10000);
            System.out.println("GC'ing...");
        }
    }

    private static final class MessageListenerImplementation implements MessageListener {
        private final Session session;
        private final Connection connection;

        private MessageListenerImplementation(Session session, Connection connection) {
            this.session = session;
            this.connection = connection;
            System.out.println("I stored the connection in the message listener");
        }

        @Override
        public void onMessage(Message message) {
            try {
                System.out.println(message);
            } catch (RuntimeException ex) {
                try {
                    ex.printStackTrace();
                    session.recover();
                } catch (JMSException recoveryException) {
                    ex.printStackTrace();
                }
            }
        }
    }

    private final static Logger log = Logger.getLogger("all");
}
